@@ -32,8 +32,8 @@ So please:
  * valid condition login should be preceded by "S=".
 By default, if no header is found the condition is assume to be a fail,
 so checking for *invalid* login.
-The fourth optional value, can be a 'C' to do *not* do GET requests before
-POST requests to obtain and replay cookies.
+The fourth optional value, can be a 'C' to define a different page to GET
+initial cookies from.
 
 If you specify the verbose flag (-v) it will show you the response from the 
 HTTP server which is useful for checking the result of a failed login to 
@@ -136,11 +136,11 @@ int analyze_server_response(int s) {
       auth_flag = 1;
     }
 
-    if (strstr(buf, "Location: ") != NULL) {
+    if (hydra_strcasestr(buf, "Location: ") != NULL) {
       char *startloc, *endloc;
       char str[2048];
 
-      startloc = strstr(buf, "Location: ") + strlen("Location: ");
+      startloc = hydra_strcasestr(buf, "Location: ") + strlen("Location: ");
       strncpy(str, startloc, sizeof(str) - 1);
       str[sizeof(str) - 1] = 0;
       endloc = strchr(str, '\n');
@@ -149,7 +149,7 @@ int analyze_server_response(int s) {
       strcpy(redirected_url_buff, str);
     }
     //there can be multiple cookies
-    if (strstr(buf, "Set-Cookie: ") != NULL) {
+    if (hydra_strcasestr(buf, "Set-Cookie: ") != NULL) {
       char *cookiebuf = buf;
 
       do {
@@ -157,7 +157,7 @@ int analyze_server_response(int s) {
         char str[2048], tmpcookie[2048] = "", tmpname[128] = "", *ptr, *ptr2;
 
         memset(str, 0, sizeof(str));
-        startcookie = strstr(cookiebuf, "Set-Cookie: ") + strlen("Set-Cookie: ");
+        startcookie = hydra_strcasestr(cookiebuf, "Set-Cookie: ") + strlen("Set-Cookie: ");
         strncpy(str, startcookie, sizeof(str) - 1);
         str[sizeof(str) - 1] = 0;
         endcookie1 = strchr(str, '\n');
@@ -174,7 +174,7 @@ int analyze_server_response(int s) {
           ptr = index(tmpname, '=');
           *(++ptr) = 0;
           // is the cookie already in the cookiejar? (so, does it have to be replaced?)
-          if ((ptr = strstr(cookie, tmpname)) != NULL) {
+          if ((ptr = hydra_strcasestr(cookie, tmpname)) != NULL) {
             // yes it is.
             // if the cookie is not in the beginning of the cookiejar, copy the ones before
             if (ptr != cookie && *(ptr - 1) == ' ') {
@@ -201,7 +201,7 @@ int analyze_server_response(int s) {
         }
         cookiebuf = startcookie;
       }
-      while (strstr(cookiebuf, "Set-Cookie: ") != NULL);
+      while (hydra_strcasestr(cookiebuf, "Set-Cookie: ") != NULL);
     }
 #ifdef HAVE_PCRE
     if (hydra_string_match(buf, cond)) {
@@ -215,7 +215,7 @@ int analyze_server_response(int s) {
   }
   if (runs == 0 && success_cond == 0)
     return !success_cond;
-  else
+  else 
     return success_cond;
 }
 
@@ -250,7 +250,7 @@ int start_http_form(int s, char *ip, int port, unsigned char options, char *misc
     // proxy with authentication
     if (getcookie) {
       //doing a GET to save cookies
-      sprintf(buffer, "GET http://%s:%d%.600s HTTP/1.0\r\nHost: %s\r\nProxy-Authorization: Basic %s\r\nUser-Agent: Mozilla/4.0 (Hydra)\r\n%s%s\r\n",
+      sprintf(buffer, "GET http://%s:%d%.600s HTTP/1.0\r\nHost: %s\r\nProxy-Authorization: Basic %s\r\nUser-Agent: Mozilla 5.0 (Hydra Proxy Auth)\r\n%s%s\r\n",
               webtarget, webport, cookieurl, webtarget, proxy_authentication, header, userheader);
       if (hydra_send(s, buffer, strlen(buffer), 0) < 0) {
         return 1;
@@ -264,14 +264,14 @@ int start_http_form(int s, char *ip, int port, unsigned char options, char *misc
 
     if (strcmp(type, "POST") == 0) {
       sprintf(buffer,
-              "POST http://%s:%d%.600s HTTP/1.0\r\nHost: %s\r\nProxy-Authorization: Basic %s\r\nUser-Agent: Mozilla/4.0 (Hydra)\r\nContent-Type: application/x-www-form-urlencoded\r\nContent-Length: %d\r\n%s%s\r\n%s",
+              "POST http://%s:%d%.600s HTTP/1.0\r\nHost: %s\r\nProxy-Authorization: Basic %s\r\nUser-Agent: Mozilla/5.0 (Hydra Proxy Auth)\r\nContent-Type: application/x-www-form-urlencoded\r\nContent-Length: %d\r\n%s%s\r\n%s",
               webtarget, webport, url, webtarget, proxy_authentication, (int) strlen(upd3variables), header, userheader, upd3variables);
       if (hydra_send(s, buffer, strlen(buffer), 0) < 0) {
         return 1;
       }
     } else {
       sprintf(buffer,
-              "GET http://%s:%d%.600s?%s HTTP/1.0\r\nHost: %s\r\nProxy-Authorization: Basic %s\r\nUser-Agent: Mozilla/4.0 (Hydra)\r\n%s%s\r\n",
+              "GET http://%s:%d%.600s?%s HTTP/1.0\r\nHost: %s\r\nProxy-Authorization: Basic %s\r\nUser-Agent: Mozilla/5.0 (Hydra Proxy Auth)\r\n%s%s\r\n",
               webtarget, webport, url, upd3variables, webtarget, proxy_authentication, header, userheader);
       if (hydra_send(s, buffer, strlen(buffer), 0) < 0) {
         return 1;
@@ -282,7 +282,7 @@ int start_http_form(int s, char *ip, int port, unsigned char options, char *misc
       // proxy without authentication
       if (getcookie) {
         //doing a GET to get cookies
-        sprintf(buffer, "GET http://%s:%d%.600s HTTP/1.0\r\nHost: %s\r\nUser-Agent: Mozilla/4.0 (Hydra)\r\n%s%s\r\n", webtarget, webport, cookieurl, webtarget, header, userheader);
+        sprintf(buffer, "GET http://%s:%d%.600s HTTP/1.0\r\nHost: %s\r\nUser-Agent: Mozilla/5.0 (Hydra Proxy)\r\n%s%s\r\n", webtarget, webport, cookieurl, webtarget, header, userheader);
         if (hydra_send(s, buffer, strlen(buffer), 0) < 0) {
           return 1;
         }
@@ -295,13 +295,13 @@ int start_http_form(int s, char *ip, int port, unsigned char options, char *misc
 
       if (strcmp(type, "POST") == 0) {
         sprintf(buffer,
-                "POST http://%s:%d%.600s HTTP/1.0\r\nHost: %s\r\nUser-Agent: Mozilla/4.0 (Hydra)\r\nContent-Type: application/x-www-form-urlencoded\r\nContent-Length: %d\r\n%s%s\r\n%s",
+                "POST http://%s:%d%.600s HTTP/1.0\r\nHost: %s\r\nUser-Agent: Mozilla/5.0 (Hydra)\r\nContent-Type: application/x-www-form-urlencoded\r\nContent-Length: %d\r\n%s%s\r\n%s",
                 webtarget, webport, url, webtarget, (int) strlen(upd3variables), header, userheader, upd3variables);
         if (hydra_send(s, buffer, strlen(buffer), 0) < 0) {
           return 1;
         }
       } else {
-        sprintf(buffer, "GET http://%s:%d%.600s?%s HTTP/1.0\r\nHost: %s\r\nUser-Agent: Mozilla/4.0 (Hydra)\r\n%s%s\r\n", webtarget, webport, url, upd3variables, webtarget, header, userheader);
+        sprintf(buffer, "GET http://%s:%d%.600s?%s HTTP/1.0\r\nHost: %s\r\nUser-Agent: Mozilla/5.0 (Hydra)\r\n%s%s\r\n", webtarget, webport, url, upd3variables, webtarget, header, userheader);
         if (hydra_send(s, buffer, strlen(buffer), 0) < 0) {
           return 1;
         }
@@ -310,7 +310,7 @@ int start_http_form(int s, char *ip, int port, unsigned char options, char *misc
       // direct web server, no proxy
       if (getcookie) {
         //doing a GET to save cookies
-        sprintf(buffer, "GET %.600s HTTP/1.0\r\nHost: %s\r\nUser-Agent: Mozilla/4.0 (Hydra)\r\n%s\r\n", cookieurl, webtarget, userheader);
+        sprintf(buffer, "GET %.600s HTTP/1.0\r\nHost: %s\r\nUser-Agent: Mozilla/5.0 (Hydra)\r\n%s\r\n", cookieurl, webtarget, userheader);
         if (hydra_send(s, buffer, strlen(buffer), 0) < 0) {
           return 1;
         }
@@ -323,13 +323,13 @@ int start_http_form(int s, char *ip, int port, unsigned char options, char *misc
 
       if (strcmp(type, "POST") == 0) {
         sprintf(buffer,
-                "POST %.600s HTTP/1.0\r\nHost: %s\r\nUser-Agent: Mozilla/4.0 (Hydra)\r\nContent-Type: application/x-www-form-urlencoded\r\nContent-Length: %d\r\n%s%s\r\n%s",
+                "POST %.600s HTTP/1.0\r\nHost: %s\r\nUser-Agent: Mozilla/5.0 (Hydra)\r\nContent-Type: application/x-www-form-urlencoded\r\nContent-Length: %d\r\n%s%s\r\n%s",
                 url, webtarget, (int) strlen(upd3variables), header, userheader, upd3variables);
         if (hydra_send(s, buffer, strlen(buffer), 0) < 0) {
           return 1;
         }
       } else {
-        sprintf(buffer, "GET http://%.600s?%s HTTP/1.0\r\nHost: %s\r\nUser-Agent: Mozilla/4.0 (Hydra)\r\n%s%s\r\n", url, upd3variables, webtarget, header, userheader);
+        sprintf(buffer, "GET %.600s?%s HTTP/1.0\r\nHost: %s\r\nUser-Agent: Mozilla/5.0 (Hydra)\r\n%s%s\r\n", url, upd3variables, webtarget, header, userheader);
         if (hydra_send(s, buffer, strlen(buffer), 0) < 0) {
           return 1;
         }
@@ -339,7 +339,7 @@ int start_http_form(int s, char *ip, int port, unsigned char options, char *misc
 
   found = analyze_server_response(s);
   if (auth_flag) {              // we received a 401 error - user using wrong module
-    hydra_report(stderr, "ERROR: the target is using HTTP auth, not a web form, received HTTP error code 401. Use module \"http%s-get\" instead.\n",
+    hydra_report(stderr, "Error: the target is using HTTP auth, not a web form, received HTTP error code 401. Use module \"http%s-get\" instead.\n",
                  (options & OPTION_SSL) > 0 ? "s" : "");
     return 4;
   }
@@ -367,12 +367,29 @@ int start_http_form(int s, char *ip, int port, unsigned char options, char *misc
       startloc = strstr(redirected_url_buff, "://");
       if (startloc != NULL) {
         startloc += strlen("://");
+
+        if ((endloc=strchr(startloc, '\r')) != NULL) {
+          startloc[endloc - startloc] = 0;
+        }
+        if ((endloc=strchr(startloc, '\n')) != NULL) {
+          startloc[endloc - startloc] = 0;
+        }
         strcpy(str, startloc);
+
         endloc = strchr(str, '/');
-        strncpy(str2, str, endloc - str);
-        str2[endloc - str] = 0;
-        strncpy(str3, str + strlen(str2), strlen(str) - strlen(str2) - 1);
-        str3[strlen(str) - strlen(str2) - 1] = 0;
+        if (endloc != NULL) {
+          strncpy(str2, str, endloc - str);
+          str2[endloc - str] = 0;
+        }
+        else
+            strncpy(str2, str, sizeof(str));
+
+        if (strlen(str) - strlen(str2) == 0) {
+          strcpy(str3, "/");
+        } else {
+          strncpy(str3, str + strlen(str2), strlen(str) - strlen(str2) - 1);
+          str3[strlen(str) - strlen(str2) - 1] = 0;
+        }
       } else {
         strncpy(str2, webtarget, sizeof(str2));
         strncpy(str3, redirected_url_buff, sizeof(str3));
@@ -494,12 +511,13 @@ void service_http_form(char *ip, int sp, unsigned char options, char *miscptr, F
   }
 
   while (1) {
-    next_run = 0;
-    if (memcmp(hydra_get_next_pair(), &HYDRA_EXIT, sizeof(HYDRA_EXIT)) == 0) {
-      if (freemischttpform)
-        free(miscptr);
-      freemischttpform = 0;
-      hydra_child_exit(1);
+    if (run == 2) {
+      if (memcmp(hydra_get_next_pair(), &HYDRA_EXIT, sizeof(HYDRA_EXIT)) == 0) {
+        if (freemischttpform)
+          free(miscptr);
+        freemischttpform = 0;
+        hydra_child_exit(1);
+      }
     }
     switch (run) {
     case 1:                    /* connect and service init function */
