@@ -1,12 +1,9 @@
 #include "hydra-mod.h"
-
 #include <arpa/inet.h>
-
 #ifdef LIBOPENSSL
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 #endif
-
 #ifdef HAVE_PCRE
 #include <pcre.h>
 #endif
@@ -15,8 +12,21 @@
 #define WAIT_BETWEEN_CONNECT_RETRY 3
 #define HYDRA_DUMP_ROWS 16
 
+/* rfc 1928 SOCKS proxy */
+#define SOCKS_V5	5
+#define SOCKS_V4	4
+#define SOCKS_NOAUTH	0
+/* http://tools.ietf.org/html/rfc1929 */
+#define SOCKS_PASSAUTH	2
+#define SOCKS_NOMETHOD	0xff
+#define SOCKS_CONNECT	1
+#define SOCKS_IPV4	1
+#define SOCKS_DOMAIN	3
+#define SOCKS_IPV6	4
+
 extern int conwait;
 
+int module_auth_type = -1;
 int intern_socket, extern_socket;
 char pair[260];
 char HYDRA_EXIT[5] = "\x00\xff\x00\xff\x00";
@@ -31,7 +41,6 @@ int __fck = 0;
 int ssl_first = 1;
 int __first_connect = 1;
 char ipstring[64];
-
 #ifdef LIBOPENSSL
 SSL *ssl = NULL;
 SSL_CTX *sslContext = NULL;
@@ -42,11 +51,9 @@ RSA *rsa = NULL;
 int my_select(int fd, fd_set * fdread, fd_set * fdwrite, fd_set * fdex, long sec, long usec);
 
 /* ----------------- alarming functions ---------------- */
-
 void alarming() {
   fail++;
   alarm_went_off++;
-
 /* uh, I think it's not good for performance if we try to reconnect to a timeout system!
  *  if (fail > MAX_CONNECT_RETRY) {
  */
@@ -54,7 +61,6 @@ void alarming() {
   if (debug)
     printf("DEBUG_CONNECT_TIMEOUT\n");
   hydra_child_exit(1);
-
 /*
  *   } else {
  *     if (verbose) fprintf(stderr, "Process %d: Can not connect [timeout], retrying (%d of %d retries)\n", (int)getpid(), fail, MAX_CONNECT_RETRY);
@@ -71,7 +77,6 @@ void interrupt() {
 
 int internal__hydra_connect(char *host, int port, int protocol, int type) {
   int s, ret = -1, ipv6 = 0;
-
 #ifdef AF_INET6
   struct sockaddr_in6 target6;
   struct sockaddr_in6 sin6;
@@ -79,6 +84,7 @@ int internal__hydra_connect(char *host, int port, int protocol, int type) {
   struct sockaddr_in target;
   struct sockaddr_in sin;
   char *buf, *tmpptr = NULL;
+  int err = 0;
 
 #ifdef AF_INET6
   memset(&target6, 0, sizeof(target6));
@@ -175,15 +181,16 @@ int internal__hydra_connect(char *host, int port, int protocol, int type) {
 #ifdef AF_INET6
 #ifdef SO_BINDTODEVICE
       if (host[17] != 0) {
-        setsockopt(s, SOL_SOCKET, SO_BINDTODEVICE, &host[17], strlen(&host[17])+1);
+        setsockopt(s, SOL_SOCKET, SO_BINDTODEVICE, &host[17], strlen(&host[17]) + 1);
       }
 #else
 #ifdef IP_FORCE_OUT_IFP
       if (host[17] != 0) {
-        setsockopt(s, SOL_SOCKET, IP_FORCE_OUT_IFP, &host[17], strlen(&host[17])+1);
+        setsockopt(s, SOL_SOCKET, IP_FORCE_OUT_IFP, &host[17], strlen(&host[17]) + 1);
       }
 #endif
 #endif
+
       if (ipv6)
         ret = connect(s, (struct sockaddr *) &target6, sizeof(target6));
       else
@@ -213,6 +220,7 @@ int internal__hydra_connect(char *host, int port, int protocol, int type) {
     if (debug)
       printf("DEBUG_CONNECT_OK\n");
 
+    err = 0;
     if (use_proxy == 2) {
       if ((buf = malloc(4096)) == NULL) {
         fprintf(stderr, "[ERROR] could not malloc()\n");
@@ -222,35 +230,190 @@ int internal__hydra_connect(char *host, int port, int protocol, int type) {
       if (host[0] == 4) {
         memcpy(&target.sin_addr.s_addr, &host[1], 4);
         target.sin_family = AF_INET;
+        target.sin_port = htons(port);
       }
 #ifdef AF_INET6
       memset(&target6, 0, sizeof(target6));
       if (host[0] == 16) {
         memcpy(&target6.sin6_addr, &host[1], 16);
         target6.sin6_family = AF_INET6;
+        target6.sin6_port = htons(port);
       }
 #endif
-      if (proxy_authentication == NULL)
-        snprintf(buf, 4096, "CONNECT %s:%d HTTP/1.0\r\n\r\n", hydra_address2string(host), port);
-      else
-        snprintf(buf, 4096, "CONNECT %s:%d HTTP/1.0\r\nProxy-Authorization: Basic %s\r\n\r\n", hydra_address2string(host), port, proxy_authentication);
-      send(s, buf, strlen(buf), 0);
-      recv(s, buf, 4096, 0);
-      if (strncmp("HTTP/", buf, 5) == 0 && (tmpptr = index(buf, ' ')) != NULL && *++tmpptr == '2') {
-        if (debug)
-          printf("DEBUG_CONNECT_PROXY_OK\n");
+
+      if (hydra_strcasestr(proxy_string_type, "connect") || hydra_strcasestr(proxy_string_type, "http")) {
+	if (proxy_authentication == NULL)
+          if (host[0] == 16)
+            snprintf(buf, 4096, "CONNECT [%s]:%d HTTP/1.0\r\n\r\n", hydra_address2string(host), port);
+          else
+            snprintf(buf, 4096, "CONNECT %s:%d HTTP/1.0\r\n\r\n", hydra_address2string(host), port);
+	else
+          if (host[0] == 16)
+            snprintf(buf, 4096, "CONNECT [%s]:%d HTTP/1.0\r\nProxy-Authorization: Basic %s\r\n\r\n", hydra_address2string(host), port, proxy_authentication);
+          else
+            snprintf(buf, 4096, "CONNECT %s:%d HTTP/1.0\r\nProxy-Authorization: Basic %s\r\n\r\n", hydra_address2string(host), port, proxy_authentication);
+
+	send(s, buf, strlen(buf), 0);
+	recv(s, buf, 4096, 0);
+	if (strncmp("HTTP/", buf, 5) == 0 && (tmpptr = index(buf, ' ')) != NULL && *++tmpptr == '2') {
+          if (debug)
+            printf("DEBUG_CONNECT_PROXY_OK\n");
+	} else {
+          if (debug)
+            printf("DEBUG_CONNECT_PROXY_FAILED (Code: %c%c%c)\n", *tmpptr, *(tmpptr + 1), *(tmpptr + 2));
+          if (verbose)
+            fprintf(stderr, "[ERROR] CONNECT call to proxy failed with code %c%c%c\n", *tmpptr, *(tmpptr + 1), *(tmpptr + 2));
+          err = 1;
+	}
+	free(buf);
       } else {
-        if (debug)
-          printf("DEBUG_CONNECT_PROXY_FAILED (Code: %c%c%c)\n", *tmpptr, *(tmpptr + 1), *(tmpptr + 2));
-        if (verbose)
-          fprintf(stderr, "[ERROR] CONNECT call to proxy failed with code %c%c%c\n", *tmpptr, *(tmpptr + 1), *(tmpptr + 2));
-        close(s);
-        extern_socket = -1;
-        ret = -1;
-        free(buf);
-        return ret;
+        if (hydra_strcasestr(proxy_string_type, "socks5")) {
+	  char buf[1024];
+	  size_t cnt, wlen;
+
+          /* socks v5 support */
+	  buf[0] = SOCKS_V5;
+	  buf[1] = 1;
+	  if (proxy_authentication == NULL)
+	    buf[2] = SOCKS_NOAUTH;
+          else
+	    buf[2] = SOCKS_PASSAUTH;
+	  cnt = hydra_send(s, buf, 3, 0);
+	  if (cnt != 3) {
+	    hydra_report(stderr, "[ERROR] SOCKS5 proxy write failed (%zu/3)\n", cnt);
+            err = 1;
+          } else {
+	    cnt = hydra_recv(s, buf, 2);
+	    if (cnt != 2) {
+	      hydra_report(stderr, "[ERROR] SOCKS5 proxy read failed (%zu/2)\n", cnt);
+              err = 1;
+            }
+	    if (buf[1] == SOCKS_NOMETHOD) {
+	      hydra_report(stderr, "[ERROR] SOCKS5 proxy authentication method negotiation failed\n");
+              err = 1;
+            }
+            /* SOCKS_DOMAIN not supported here, do we need it ? */
+            if (err != 1) {
+              /* send user/pass */
+              if (proxy_authentication != NULL) {
+                //format was checked previously
+                char *login=strtok(proxy_authentication, ":");
+                char *pass=strtok(NULL, ":");
+                snprintf(buf, sizeof(buf), "\x01%c%s%c%s", (char) strlen(login), login, (char) strlen(pass), pass);
+
+		cnt = hydra_send(s, buf, strlen(buf), 0);
+		if (cnt != strlen(buf)) {
+		  hydra_report(stderr, "[ERROR] SOCKS5 proxy write failed (%zu/3)\n", cnt);
+        	  err = 1;
+        	} else {
+		  cnt = hydra_recv(s, buf, 2);
+		  if (cnt != 2) {
+		    hydra_report(stderr, "[ERROR] SOCKS5 proxy read failed (%zu/2)\n", cnt);
+        	    err = 1;
+        	  }
+        	  if (buf[1] != 0) {
+		    hydra_report(stderr, "[ERROR] SOCKS5 proxy authentication failure\n");
+        	    err = 1;
+        	  } else {
+                    if (debug)
+  		      hydra_report(stderr, "[DEBUG] SOCKS5 proxy authentication success\n");
+        	  }
+        	}
+              }
+
+#ifdef AF_INET6
+              if (ipv6) {
+		/* Version 5, connect: IPv6 address */
+		buf[0] = SOCKS_V5;
+		buf[1] = SOCKS_CONNECT;
+		buf[2] = 0;
+		buf[3] = SOCKS_IPV6;
+		memcpy(buf + 4, &target6.sin6_addr, sizeof target6.sin6_addr);
+		memcpy(buf + 20, &target6.sin6_port, sizeof target6.sin6_port);
+		wlen = 22;
+              } else {
+#endif
+		/* Version 5, connect: IPv4 address */
+		buf[0] = SOCKS_V5;
+		buf[1] = SOCKS_CONNECT;
+		buf[2] = 0;
+		buf[3] = SOCKS_IPV4;
+		memcpy(buf + 4, &target.sin_addr, sizeof target.sin_addr);
+		memcpy(buf + 8, &target.sin_port, sizeof target.sin_port);
+		wlen = 10;
+#ifdef AF_INET6
+              }
+#endif
+	      cnt = hydra_send(s, buf, wlen, 0);
+	      if (cnt != wlen) {
+ 		hydra_report(stderr, "[ERROR] SOCKS5 proxy write failed (%zu/%zu)\n", cnt, wlen);
+        	err = 1;
+              } else {
+		cnt=hydra_recv(s, buf, 10);
+		if (cnt != 10) {
+        	  hydra_report(stderr, "[ERROR] SOCKS5 proxy read failed (%zu/10)\n", cnt);
+        	  err = 1;
+        	}
+		if (buf[1] != 0) {
+                  /* 0x05 = connection refused by destination host */
+                  if (buf[1] == 5)
+            	    hydra_report(stderr, "[ERROR] SOCKS proxy request failed\n");
+                  else
+                    hydra_report(stderr, "[ERROR] SOCKS error %d\n", buf[1]);
+        	  err = 1;
+        	}
+              }
+            }
+          }
+        } else {
+          if (hydra_strcasestr(proxy_string_type, "socks4")) {
+	    if (ipv6) {
+              hydra_report(stderr, "[ERROR] SOCKS4 proxy does not support IPv6\n");
+              err = 1;
+	    } else {
+	      char buf[1024];
+	      size_t cnt, wlen;
+
+              /* socks v4 support */
+	      buf[0] = SOCKS_V4;
+	      buf[1] = SOCKS_CONNECT; /* connect */
+	      memcpy(buf + 2, &target.sin_port, sizeof target.sin_port);
+	      memcpy(buf + 4, &target.sin_addr, sizeof target.sin_addr);
+	      buf[8] = 0;     /* empty username */
+	      wlen = 9;
+	      cnt = hydra_send(s, buf, wlen, 0); 
+	      if (cnt != wlen) {
+		hydra_report(stderr, "[ERROR] SOCKS4 proxy write failed (%zu/%zu)\n", cnt, wlen);
+        	err = 1;
+              } else {
+		cnt=hydra_recv(s, buf, 8);
+        	if (cnt != 8) {
+        	  hydra_report(stderr, "[ERROR] SOCKS4 proxy read failed (%zu/8)\n", cnt);
+        	  err = 1;
+        	}
+        	if (buf[1] != 90) {
+                  /* 91 = 0x5b = request rejected or failed */
+                  if (buf[1] == 91)
+            	    hydra_report(stderr, "[ERROR] SOCKS proxy request failed\n");
+                  else
+        	    hydra_report(stderr, "[ERROR] SOCKS error %d\n", buf[1]);
+        	  err = 1;
+        	}
+	      }
+            }
+	  } else {
+              hydra_report(stderr, "[ERROR] Unknown proxy type: %s, valid type are \"connect\", \"socks4\" or \"socks5\"\n", proxy_string_type);
+              err = 1;
+	  }
+        }
       }
+    }
+    if (err) {
+      close(s);
+      extern_socket = -1;
+      ret = -1;
       free(buf);
+      return ret;
     }
     fail = 0;
     return ret;
@@ -353,13 +516,14 @@ int internal__hydra_send(int socket, char *buf, int size, int options) {
 
 void hydra_child_exit(int code) {
   char buf[2];
+
   if (debug)
     fprintf(stderr, "[DEBUG] pid %d called child_exit with code %d\n", getpid(), code);
   if (code == 0)                /* normal quitting */
     __fck = write(intern_socket, "Q", 1);
-  else if (code == 1)                /* no connect possible */
+  else if (code == 1)           /* no connect possible */
     __fck = write(intern_socket, "C", 1);
-  else if (code == 2)                /* application protocol error or service shutdown */
+  else if (code == 2)           /* application protocol error or service shutdown */
     __fck = write(intern_socket, "E", 1);
   // code 3 means exit without telling mommy about it - a bad idea. mommy should know
   else if (code == -1 || code > 3) {
@@ -370,7 +534,7 @@ void hydra_child_exit(int code) {
     usleep(10000);
   } while (read(intern_socket, buf, 1) <= 0);
 //  sleep(2); // be sure that mommy receives our message
-  exit(0);  // might be killed before reaching this
+  exit(0);                      // might be killed before reaching this
 }
 
 void hydra_register_socket(int s) {
@@ -386,7 +550,6 @@ char *hydra_get_next_pair() {
     if (pair[0] == 0)
       return HYDRA_EMPTY;
   }
-//sleep(10);
   return pair;
 }
 
@@ -433,40 +596,37 @@ void hydra_completed_pair_skip() {
 /*
 based on writeError from Medusa project
 */
-void
-hydra_report_debug(FILE *st, char *format, ...)
-{
+void hydra_report_debug(FILE * st, char *format, ...) {
   va_list ap;
   char buf[512];
-  char bufOut[1024];
+  char bufOut[2048];
   char temp[6];
   unsigned char cTemp;
   int i = 0;
- 
+
   if (format == NULL) {
     fprintf(stderr, "[ERROR] no msg passed.\n");
-  }
-  else {
+  } else {
     if (format[strlen(format) - 1] == '\n')
       format[strlen(format) - 1] = 0;
     va_start(ap, format);
-    memset(bufOut, 0, 1024);
+    memset(bufOut, 0, sizeof(bufOut));
     memset(buf, 0, 512);
     vsnprintf(buf, sizeof(buf), format, ap);
- 
+
     // Convert any chars less than 32d or greater than 126d to hex
-    for (i = 0; i < sizeof(buf); i++)
-    {
+    for (i = 0; i < sizeof(buf); i++) {
       memset(temp, 0, 6);
-      cTemp = (unsigned char)buf[i];
-      if ((cTemp < 32 && cTemp > 0) || cTemp > 126)
-      {
-        sprintf(temp, "[%02X]", cTemp);        
-      }
-      else
+      cTemp = (unsigned char) buf[i];
+      if ((cTemp < 32 && cTemp > 0) || cTemp > 126) {
+        sprintf(temp, "[%02X]", cTemp);
+      } else
         sprintf(temp, "%c", cTemp);
-      
-      strncat(bufOut, temp, 6);
+
+      if (strlen(bufOut) + 6 < sizeof(bufOut))
+        strncat(bufOut, temp, 6);
+      else
+        break;
     }
     fprintf(st, "%s\n", bufOut);
     va_end(ap);
@@ -482,7 +642,6 @@ void hydra_report_found(int port, char *svc, FILE * fp) {
   fflush(fp);
 }
 
-
 /* needed for irc module to display the general server password */
 void hydra_report_pass_found(int port, char *ip, char *svc, FILE * fp) {
   strcpy(ipaddr_str, hydra_address2string(ip));
@@ -597,52 +756,67 @@ int hydra_recv(int socket, char *buf, int length) {
 
   ret = internal__hydra_recv(socket, buf, length);
   if (debug)
-    hydra_report_debug(stderr, "DEBUG_RECV_BEGIN|%s|END [pid:%d]", buf, getpid());
+    hydra_report_debug(stderr, "DEBUG_RECV_BEGIN|%s|END [pid:%d ret:%d]", buf, getpid(), ret);
   return ret;
 }
 
 char *hydra_receive_line(int socket) {
-  char buf[300], *buff;
-  int i = 0, j = 0, k;
+  char buf[1024], *buff;
+  int i, j = 1, k, got = 0;
 
   if ((buff = malloc(sizeof(buf))) == NULL) {
     fprintf(stderr, "[ERROR] could not malloc\n");
     return NULL;
   }
   memset(buff, 0, sizeof(buf));
+  if (debug)
+    printf("[DEBUG] hydra_receive_line: waittime: %d, socket: %d\n", waittime, socket);
 
-  i = hydra_data_ready_timed(socket, (long) waittime, 0);
-  if (i > 0) {
-    if ((i = internal__hydra_recv(socket, buff, sizeof(buf))) < 0) {
+  if ((i = hydra_data_ready_timed(socket, (long) waittime, 0)) > 0) {
+    if ((got = internal__hydra_recv(socket, buff, sizeof(buf))) < 0) {
       free(buff);
       return NULL;
     }
-  }
-  if (i <= 0) {
+  } else {
     if (debug)
-      hydra_report_debug(stderr, "DEBUG_RECV_BEGIN|%s|END [pid:%d]", buff, getpid());
+      printf("[DEBUG] hydra_data_ready_timed: %d, waittime: %d\n", i, waittime);
+    i = 0;
+  }
+
+  if (got < 0) {
+    if (debug) {
+      hydra_report_debug(stderr, "DEBUG_RECV_BEGIN||END [pid:%d %d]", getpid(), i);
+      perror("recv");
+    }
     free(buff);
     return NULL;
   } else {
-    for (k = 0; k < i; k++)
-      if (buff[k] == 0)
-        buff[k] = 32;
+      if (got > 0) {
+        for (k = 0; k < i; k++)
+          if (buff[k] == 0)
+            buff[k] = 32;
+      }
   }
 
-  j = 1;
   while (hydra_data_ready(socket) > 0 && j > 0) {
     j = internal__hydra_recv(socket, buf, sizeof(buf));
-    if (j > 0)
+    if (j > 0) {
       for (k = 0; k < j; k++)
         if (buf[k] == 0)
           buf[k] = 32;
-    buff = realloc(buff, i + j);
-    memcpy(buff + i, &buf, j);
-    i = i + j;
+      buff = realloc(buff, got + j + 1);
+      memcpy(buff + got, &buf, j);
+      got += j;
+      buff[got] = 0;
+    }
   }
 
   if (debug)
-    hydra_report_debug(stderr, "DEBUG_RECV_BEGIN|%s|END [pid:%d]", buff, getpid());
+    hydra_report_debug(stderr, "DEBUG_RECV_BEGIN [pid:%d len:%d]|%s|END", getpid(), got, buff);
+  if (got == 0) {
+    free(buff);
+    return NULL;
+  }
   return buff;
 }
 
@@ -662,7 +836,6 @@ int hydra_send(int socket, char *buf, int size, int options) {
       free(debugbuf);
     }
   }
-
 /*    if (hydra_data_ready_writing(socket)) < 1) return -1; XXX maybe needed in the future */
   return (internal__hydra_send(socket, buf, size, options));
 }
@@ -824,7 +997,6 @@ void hydra_set_srcport(int port) {
 
 #ifdef HAVE_PCRE
 int hydra_string_match(char *str, const char *regex) {
-
   pcre *re = NULL;
   int offset_error = 0;
   const char *error = NULL;
@@ -854,7 +1026,6 @@ int hydra_string_match(char *str, const char *regex) {
  *
  */
 char *hydra_string_replace(const char *string, const char *substr, const char *replacement) {
-
   char *tok = NULL;
   char *newstr = NULL;
 
@@ -870,3 +1041,24 @@ char *hydra_string_replace(const char *string, const char *substr, const char *r
   memset(newstr + strlen(string) - strlen(substr) + strlen(replacement), 0, 1);
   return newstr;
 }
+
+char *hydra_strcasestr(const char *haystack, const char *needle) {
+  if (needle == NULL || *needle == 0)
+    return NULL;
+
+  for (; *haystack; ++haystack) {
+    if (toupper((int)*haystack) == toupper((int)*needle)) {
+      const char *h, *n;
+
+      for (h = haystack, n = needle; *h && *n; ++h, ++n) {
+        if (toupper((int)*h) != toupper((int)*n)) {
+          break;
+        }
+      }
+      if (!*n) {                /* matched all of 'needle' to null termination */
+        return (char *) haystack;       /* return the start of the match */
+      }
+    }
+  }
+  return NULL;
+}
