@@ -103,9 +103,10 @@ http://technet.microsoft.com/en-us/library/cc960646.aspx
 
 
 extern char *HYDRA_EXIT;
-unsigned char challenge[8];
-unsigned char workgroup[16];
-unsigned char machine_name[16];
+static unsigned char challenge[8];
+static unsigned char workgroup[16];
+static unsigned char domain[16];
+static unsigned char machine_name[16];
 int hashFlag, accntFlag, protoFlag;
 
 int smb_auth_mechanism=AUTH_NTLM;
@@ -186,7 +187,6 @@ int HashLM(unsigned char **lmhash, unsigned char *pass, unsigned char *challenge
       p++;
     }
 
-    p = pass;
     if (*p == '\0') {
       hydra_report(stderr, "Error reading PwDump file.\n");
       return -1;
@@ -281,7 +281,7 @@ int MakeNTLM (unsigned char *ntlmhash, unsigned char *pass)
   if (hashFlag == 1) {
     /* 1000:D42E35E1A1E4C22BD32E2170E4857C20:5E20780DD45857A68402938C7629D3B2::: */
     p = pass;
-    while ((*p != '\0') && (i < 2)) {
+    while ((*p != '\0') && (i < 1)) {
       if (*p == ':')
         i++;
       p++;
@@ -629,7 +629,7 @@ int HashNTLM(unsigned char **ntlmhash, unsigned char *pass, unsigned char *chall
 
   ret = MakeNTLM((unsigned char *)&hash, (unsigned char *)pass);
   if (ret == -1)
-    hydra_child_exit(0);;
+    hydra_child_exit(0);
 
   memset(p21, '\0', 21);
   memcpy(p21, hash, 16);
@@ -838,14 +838,23 @@ SMBNegProt(int s)
   memset(workgroup, 0, 16);
   memset(machine_name, 0, 16);
 
-  while ((rbuf[iResponseOffset + 8 + i * 2] != 0) && (i < 16)) {
-    workgroup[i] = rbuf[iResponseOffset + 8 + i * 2];
-    i++;
-  }
-  
-  while ((rbuf[iResponseOffset + 8 + (i + j + 1) * 2] != 0) && (j < 16)) {
-    machine_name[j] = rbuf[iResponseOffset + 8 + (i + j + 1) * 2];
-    j++;
+  //seems using LM only the domain is returned not the server
+  //and the domain is not padded with null chars
+  if (smb_auth_mechanism == AUTH_LM) {
+    while ((rbuf[iResponseOffset + 8 + i] != 0) && (i < 16)) {
+	workgroup[i] = rbuf[iResponseOffset + 8 + i];
+      i++;
+    }
+  } else {
+    while ((rbuf[iResponseOffset + 8 + i * 2] != 0) && (i < 16)) {
+	workgroup[i] = rbuf[iResponseOffset + 8 + i * 2];
+      i++;
+    }
+
+    while ((rbuf[iResponseOffset + 8 + (i + j + 1) * 2] != 0) && (j < 16)) {
+      machine_name[j] = rbuf[iResponseOffset + 8 + (i + j + 1) * 2];
+      j++;
+    }
   }
 
   if (verbose) {
@@ -877,16 +886,18 @@ unsigned long SMBSessionSetup(int s, char* szLogin, char* szPassword, char *misc
   int ret;
   int iByteCount, iOffset=0;
   
-  //if (_psSessionData->accntFlag == LOCAL) {
-//    strcpy((char *)workgroup, "localhost");
-memset(workgroup, 0, 16);
-/*
-  } else if (_psSessionData->accntFlag == BOTH) {
-    memset(_psSessionData->workgroup, 0, 16);
-  } else if (_psSessionData->accntFlag == OTHER) {
-    strncpy((char *) _psSessionData->workgroup, _psSessionData->workgroup_other, 16);
+  if (accntFlag == 0) {
+    strcpy((char *)workgroup, "localhost");
+
+  } else if (accntFlag == 2) {
+    memset(workgroup, 0, 16);
   }
-*/
+  //domain flag is not needed here, it will be auto set,
+  //below it's domain specified on cmd line
+  else if (accntFlag == 4) {
+    strncpy((char *)workgroup, (char *)domain, 16);
+  }
+
   /* NetBIOS Session Service */
   unsigned char szNBSS[4] = {
     0x00,                                             /* Message Type: Session Message */
@@ -1270,11 +1281,37 @@ service_smb(char *ip, int sp, unsigned char options, char *miscptr, FILE * fp, i
   if (miscptr) {
     //check group
     strupper(miscptr);
-    if (strstr(miscptr, "LOCAL") != NULL) {
+    if (strstr(miscptr, "OTHER_DOMAIN:") != NULL) {
+      char *tmpdom;
+      int err = 0;
+
+      accntFlag = 4; //OTHER DOMAIN
+      tmpdom = strstr(miscptr, "OTHER_DOMAIN:");
+      tmpdom = tmpdom + strlen("OTHER_DOMAIN:");
+     
+      if (tmpdom) {
+        //split the string after the domain if there are other values
+        strtok(tmpdom, " ");
+        if (tmpdom) {
+          strncpy((char *)domain, (char *)tmpdom, 16);
+        } else {
+          err = 1;
+        }
+      } else {
+        err = 1;
+      }
+
+      if (err) {
+        if (verbose)
+          hydra_report(stdout, "[VERBOSE] requested line mode\n");
+        accntFlag = 2;
+      }
+    } else if (strstr(miscptr, "LOCAL") != NULL) {
       accntFlag = 0; //LOCAL
     } else if (strstr(miscptr, "DOMAIN") != NULL) {
       accntFlag = 1; //DOMAIN
     }
+
     //check pass
     if (strstr(miscptr, "HASH") != NULL) {
 	  hashFlag = 1;
@@ -1292,6 +1329,10 @@ service_smb(char *ip, int sp, unsigned char options, char *miscptr, FILE * fp, i
 	  smb_auth_mechanism=AUTH_LM;
     }
   }
+  if (verbose) {
+    hydra_report(stdout, "[VERBOSE] accntFlag is %d\n", accntFlag);
+    hydra_report(stdout, "[VERBOSE] hashFlag is %d\n", accntFlag);
+  }
 
   hydra_register_socket(sp);
   if (memcmp(hydra_get_next_pair(), &HYDRA_EXIT, sizeof(HYDRA_EXIT)) == 0)
